#!/usr/bin/env bash

# Last updated 2023-01-25
#   - Ignoring bmc-sdk-java-client-parent as parent of dependencies
#   - Unless -s for strict is supplied as argument
# Updated 2023-01-20
#   - Look for outdated or version-mixed bmc-sdk-java-client-parent as well

MINIMUM_VERSION="2.38.0"
STRICT=false

vercomp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

while getopts 's' opt; do
  case "$opt" in
    s)
      STRICT=true
      ;;

    h)
      echo "Usage: $(basename $0) [-s]"
      echo " -s : strict check, also warns if dependencies have outdated bmc-sdk-java-client-parent parents"
      exit 0
      ;;

    ?)
      echo "Invalid command option."
      echo "Usage: $(basename $0) [-s]"
      echo " -s : strict check, also warns if dependencies have outdated bmc-sdk-java-client-parent parents"
      exit 1
      ;;
  esac
done
shift "$(($OPTIND -1))"

tempfile=`mktemp`
tempfile2=`mktemp`

if [ "$STRICT" = true ] ; then
    if ! mvn dependency:list -DincludeParents > $tempfile2; then
        echo "ERROR: Failed to run Maven. Are you sure you are using Maven for your project, and are you in the right directory?"
        echo ""
        cat $tempfile2
        rm $tempfile2
    fi
else
    if ! mvn dependency:list > $tempfile2; then
        echo "ERROR: Failed to run Maven. Are you sure you are using Maven for your project, and are you in the right directory?"
        echo ""
        cat $tempfile2
        rm $tempfile2
    fi
    if ! mvn org.apache.maven.plugins:maven-dependency-plugin:3.5.0:display-ancestors >> $tempfile2; then
        echo "ERROR: Failed to run Maven. Are you sure you are using Maven for your project, and are you in the right directory?"
        echo ""
        cat $tempfile2
        rm $tempfile2
    fi
fi

cat $tempfile2 | grep -o "oci-java-sdk-.*:.*:" | sed "s/:$//" | sort | uniq | tee $tempfile
cat $tempfile2 | grep -o "bmc-sdk-java-client-.*:.*$" | sed 's_^\([^<]*\) <-.*_\1_' | sed "s/:$//" | sort | uniq | tee -a $tempfile

versions=`cut -d ':' -f 3 $tempfile | sort | uniq`

if (( `echo $versions | wc -w` > 1 )); then
    echo "ERROR: You are mixing versions."
fi

for v in $versions; do
    vercomp "$MINIMUM_VERSION" $v
    if (( $? == 1 )); then
        echo "ERROR: Version $v is older than $MINIMUM_VERSION."
    fi
done

rm $tempfile
rm $tempfile2